/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.example;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import junit.framework.Assert;

import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition.ProcessState;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.BonitaException;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.novabpm.util.Misc;
import org.ow2.novabpm.util.SimpleCallbackHandler;
import org.ow2.novabpm.util.StandardCallbackHandler;


public abstract class AbstractExample {
  private LoginContext loginContext;

  public abstract String getProcessId();
  public abstract void afterInstantiate(ProcessInstanceUUID instanceUUID,
      ProcessDefinitionUUID processUUID, RuntimeAPI runtimeAPI, QueryRuntimeAPI queryRuntimeAPI,
      QueryDefinitionAPI queryDefinitionAPI) throws BonitaException;

  public void execute(RuntimeAPI runtimeAPI, QueryRuntimeAPI queryRuntimeAPI,
      ManagementAPI managementAPI, QueryDefinitionAPI queryDefinitionAPI, String barFile,
      Map<String, Object> variables) throws BonitaException, IOException {
    ProcessDefinition process = deploy(managementAPI, barFile).get(getProcessId());
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    Misc.badStateIfNull(packageUUID, "packageUUID is null in process : " + process.getProcessId());
    ProcessInstanceUUID instanceUUID = launch(variables, runtimeAPI, queryRuntimeAPI, queryDefinitionAPI, process.getUUID());
    managementAPI.undeploy(packageUUID);
    Assert.assertEquals(0, queryDefinitionAPI.getProcesses(getProcessId(), ProcessState.DEPLOYED).size());
  }

  public void login() throws LoginException {
    String loginMode = System.getProperty("login.mode");
    if (loginMode != null && "test".equals(loginMode)) {
      loginContext = new LoginContext("Bonita", new SimpleCallbackHandler("John", "bonita"));
    } else {
      loginContext = new LoginContext("Bonita", new StandardCallbackHandler());
    }
    loginContext.login();
  }
  public void logout() throws LoginException {
    loginContext.logout();
  }

  public ProcessInstanceUUID launch(Map<String, Object> variables,
      RuntimeAPI runtimeAPI, QueryRuntimeAPI queryRuntimeAPI, QueryDefinitionAPI queryDefinitionAPI,
      ProcessDefinitionUUID processUUID) throws BonitaException {
    final ProcessInstanceUUID instanceUUID = runtimeAPI.instantiateProcess(processUUID, variables);
    afterInstantiate(instanceUUID, processUUID, runtimeAPI, queryRuntimeAPI,  queryDefinitionAPI);
    return instanceUUID;
  }

  protected Map<String, ProcessDefinition> deploy(ManagementAPI managementAPI,
      String barFile) throws BonitaException, IOException {
    managementAPI.clearHistory();
    if (barFile == null) {
      throw new BonitaRuntimeException("bar file is null!");
    }
    final InputStream in = this.getClass().getResourceAsStream(barFile);
    if (in == null) {
      throw new BonitaRuntimeException("Can't find ressource: " + barFile);
    }
    byte[] bar = Misc.getAllContentFrom(in);
    return managementAPI.deployBar(bar);
  }

}


