/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.example.aw;

import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.bonita.example.AbstractExample;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.bonita.util.BonitaException;
import org.ow2.bonita.util.BonitaRuntimeException;

/**
 * @author "Pierre Vigneras", "Marc Blachon", "Charles Souillard"
 */
public class ApprovalWorkflow extends AbstractExample {

  private static final Logger LOG = Logger.getLogger(ApprovalWorkflow.class.getName());

  @Override
  public String getProcessId() {
    return "ApprovalWorkflow";
  }
  @Override
  public void afterInstantiate(ProcessInstanceUUID instanceUUID,
      ProcessDefinitionUUID processUUID,
      RuntimeAPI runtimeAPI, QueryRuntimeAPI queryRuntimeAPI,
      QueryDefinitionAPI queryDefinitionAPI) throws BonitaException {
    final Collection<ActivityInstance< TaskInstance >> activities = queryRuntimeAPI.getTaskList(instanceUUID, TaskState.READY);

    if (activities.isEmpty()) {
      throw new BonitaRuntimeException("No task found? Bad User?");
    }
    for (ActivityInstance< TaskInstance > activity : activities) {
      TaskUUID taskUUID = activity.getBody().getUUID();
      String activityId = activity.getActivityId();
      if (LOG.isLoggable(Level.INFO)) {
        LOG.fine("\n***** Starting task associated to activity: " + activityId + " *****\n");
      }
      runtimeAPI.startTask(taskUUID);
      if (LOG.isLoggable(Level.INFO)) {
        LOG.fine("\n***** Finishing task associated to activity: " + activityId + " *****\n");
      }
      runtimeAPI.finishTask(taskUUID);
      if (LOG.isLoggable(Level.INFO)) {
        LOG.fine("\n***** Task associated to activity: " + activityId + " finished. *****\n");
      }
    }
  }

  public static void main(String[] args) throws Exception {
    if (args == null || args.length != 1) {
      System.err.println("Usage: " + ApprovalWorkflow.class + " bpar file");
      throw new BonitaRuntimeException("Please read the above usage");
    }
    String barFile = args[0];

    ApprovalWorkflow approvalWorkflow = new ApprovalWorkflow();
    approvalWorkflow.login();
    approvalWorkflow.execute(
        AccessorUtil.getRuntimeAPI(),
        AccessorUtil.getQueryRuntimeAPI(),
        AccessorUtil.getManagementAPI(),
        AccessorUtil.getQueryDefinitionAPI(),
        barFile, null);
    approvalWorkflow.logout();
  }

}
